/*
 * Copyright (C) 2018 Gravity Engineering Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.graveng.rci.server.config;

import com.graveng.rci.util.Serializer;

public class ServerConfig extends Serializer {
	private int port = 5243;
	protected boolean serverCertification = true;
	protected String password;

	protected String hashAlgorithmName = "SHA-256";
	protected int passwordRegistHashRoundCount = 1;
	protected int passwordAuthHashRoundCount = 0;

	protected int logLevel = 4;
	protected String logFileName = "logfile.txt";

	protected long loginWatchingTimeMillis = 60*60*1000;
	protected long loginRejectTimeMillis = 30*60*1000;
	protected int loginLimitFailCount = 10;
	protected String initializeClass = null;
	protected int aesCryptCount = 2;

	/**
	 * @return aesCryptCount
	 */
	public int getAesCryptCount() {
		return aesCryptCount;
	}

	/**
	 * @param aesCryptCount セットする aesCryptCount
	 */
	public void setAesCryptCount(int aesCryptCount) {
		this.aesCryptCount = aesCryptCount;
	}

	/**
	 * @return initializeClass
	 */
	public String getInitializeClass() {
		return initializeClass;
	}

	/**
	 * @param initializeClass セットする initializeClass
	 */
	public void setInitializeClass(String initializeClass) {
		this.initializeClass = initializeClass;
	}

	/**
	 * クライアントにサーバで発生した例外をそのまま送るならfalse
	 * 例外のクラス名とメッセージをRciExceptionのメッセージにして送るならtrueにする。
	 */
	protected boolean wrappedException = false;

	/**
	 * @return wrappedException
	 */
	public boolean isWrappedException() {
		return wrappedException;
	}

	/**
	 * @param wrappedException セットする wrappedException
	 */
	public void setWrappedException(boolean wrappedException) {
		this.wrappedException = wrappedException;
	}

	/**
	 * @return loginWatchingTimeMillis
	 */
	public long getLoginWatchingTimeMillis() {
		return loginWatchingTimeMillis;
	}

	/**
	 * @param loginWatchingTimeMillis セットする loginWatchingTimeMillis
	 */
	public void setLoginWatchingTimeMillis(long loginWatchingTimeMillis) {
		this.loginWatchingTimeMillis = loginWatchingTimeMillis;
	}

	/**
	 * @return loginRejectTimeMillis
	 */
	public long getLoginRejectTimeMillis() {
		return loginRejectTimeMillis;
	}

	/**
	 * @param loginRejectTimeMillis セットする loginRejectTimeMillis
	 */
	public void setLoginRejectTimeMillis(long loginRejectTimeMillis) {
		this.loginRejectTimeMillis = loginRejectTimeMillis;
	}

	/**
	 * @return loginLimitFailCount
	 */
	public int getLoginLimitFailCount() {
		return loginLimitFailCount;
	}

	/**
	 * @param loginLimitFailCount セットする loginLimitFailCount
	 */
	public void setLoginLimitFailCount(int loginLimitFailCount) {
		this.loginLimitFailCount = loginLimitFailCount;
	}

	/**
	 * @return logLevel
	 */
	public int getLogLevel() {
		return logLevel;
	}

	/**
	 * @param logLevel セットする logLevel
	 */
	public void setLogLevel(int logLevel) {
		this.logLevel = logLevel;
	}

	/**
	 * @return logFileName
	 */
	public String getLogFileName() {
		return logFileName;
	}

	/**
	 * @param logFileName セットする logFileName
	 */
	public void setLogFileName(String logFileName) {
		this.logFileName = logFileName;
	}

	/**
	 * @return port
	 */
	public int getPort() {
		return port;
	}

	/**
	 * @param port セットする port
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * @return serverCertification
	 */
	public boolean isServerCertification() {
		return serverCertification;
	}

	/**
	 * @param serverCertification セットする serverCertification
	 */
	public void setServerCertification(boolean serverCertification) {
		this.serverCertification = serverCertification;
	}

	/**
	 * @return password
	 */
	public String getPassword() {
		return password;
	}

	/**
	 * @param password セットする password
	 */
	public void setPassword(String password) {
		this.password = password;
	}

	/**
	 * @return hashAlgorithmName
	 */
	public String getHashAlgorithmName() {
		return hashAlgorithmName;
	}

	/**
	 * @param hashAlgorithmName セットする hashAlgorithmName
	 */
	public void setHashAlgorithmName(String hashAlgorithmName) {
		this.hashAlgorithmName = hashAlgorithmName;
	}

	/**
	 * @return passwordRegistHashRoundCount
	 */
	public int getPasswordRegistHashRoundCount() {
		return passwordRegistHashRoundCount;
	}

	/**
	 * @param passwordRegistHashRoundCount セットする passwordRegistHashRoundCount
	 */
	public void setPasswordRegistHashRoundCount(int passwordRegistHashRoundCount) {
		this.passwordRegistHashRoundCount = passwordRegistHashRoundCount;
	}

	/**
	 * @return passwordAuthHashRoundCount
	 */
	public int getPasswordAuthHashRoundCount() {
		return passwordAuthHashRoundCount;
	}

	/**
	 * @param passwordAuthHashRoundCount セットする passwordAuthHashRoundCount
	 */
	public void setPasswordAuthHashRoundCount(int passwordAuthHashRoundCount) {
		this.passwordAuthHashRoundCount = passwordAuthHashRoundCount;
	}

}
