/*
 * Copyright (C) 2018 Gravity Engineering Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.graveng.rci.util;

import java.lang.reflect.Method;

import android.util.Log;

/**
 * android環境で動かす場合はandroidの標準的なログシステムを使いたいが、
 * 非android環境でもログを出せるようにこのクラスで中継している。
 * (この構想は構造を難しくしているので、やめた方がいいかも知れない。)
 *
 */
public class Logger {
//	private static boolean isAndroid = isAndroid();
//
//	// https://developer.android.com/reference/java/lang/System.html
//	private static boolean isAndroid() {
//		String name = System.getProperty("java.vendor").toLowerCase();
//		if(name.indexOf("android") > 0) {
//			return true;
//		}
//		return false;
//	}

	protected static int level = Log.INFO;

	/**
	 * staticのみのクラス
	 */
	private Logger() {

	}

	/**
	 * VERBOSEログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void v(String msg) {
    	if(level <= Log.VERBOSE) {
            Log.v(getTag(), msg);
    	}
    }

	/**
	 * VERBOSEログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void v(String msg, Throwable tt) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg, tt);
    	}
    }

	/**
	 * VERBOSEログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void verbose(String msg) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg);
    	}
    }

	/**
	 * VERBOSEログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void verbose(String msg, Throwable tt) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg, tt);
    	}
    }

	/**
	 * TRACEログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void t(String msg) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg);
    	}
    }

	/**
	 * TRACEログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void t(String msg, Throwable tt) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg, tt);
    	}
    }

	/**
	 * TRACEログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void trace(String msg) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg);
    	}
    }

	/**
	 * TRACEログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void trace(String msg, Throwable tt) {
    	if(level <= Log.VERBOSE) {
    		Log.v(getTag(), msg, tt);
    	}
    }

	/**
	 * DEBUGログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void d(String msg) {
    	if(level <= Log.DEBUG) {
    		Log.d(getTag(), msg);
    	}
    }

	/**
	 * DEBUGログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void d(String msg, Throwable tt) {
    	if(level <= Log.DEBUG) {
    		Log.d(getTag(), msg, tt);
    	}
    }

	/**
	 * DEBUGログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void debug(String msg) {
    	if(level <= Log.DEBUG) {
    		Log.d(getTag(), msg);
    	}
    }

	/**
	 * DEBUGログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void debug(String msg, Throwable tt) {
    	if(level <= Log.DEBUG) {
    		Log.d(getTag(), msg, tt);
    	}
    }

	/**
	 * INFOログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void i(String msg) {
    	if(level <= Log.INFO) {
    		Log.i(getTag(), msg);
    	}
    }

	/**
	 * INFOログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void i(String msg, Throwable tt) {
    	if(level <= Log.INFO) {
    		Log.i(getTag(), msg, tt);
    	}
    }

	/**
	 * INFOログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void info(String msg) {
    	if(level <= Log.INFO) {
    		Log.i(getTag(), msg);
    	}
    }

	/**
	 * INFOログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void info(String msg, Throwable tt) {
    	if(level <= Log.INFO) {
    		Log.i(getTag(), msg, tt);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void w(String msg) {
    	if(level <= Log.WARN) {
    		Log.w(getTag(), msg);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void w(String msg, Throwable tt) {
    	if(level <= Log.WARN) {
    		Log.w(getTag(), msg, tt);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void warn(String msg) {
    	if(level <= Log.WARN) {
    		Log.w(getTag(), msg);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void warn(String msg, Throwable tt) {
    	if(level <= Log.WARN) {
    		Log.w(getTag(), msg, tt);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void e(String msg) {
    	if(level <= Log.ERROR) {
    		Log.e(getTag(), msg);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void e(String msg, Throwable tt) {
    	if(level <= Log.ERROR) {
    		Log.e(getTag(), msg, tt);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 */
    public static void error(String msg) {
    	if(level <= Log.ERROR) {
    		Log.e(getTag(), msg);
    	}
    }

	/**
	 * WARNログを出力する。
	 * @param msg ログメッセージ
	 * @param tt 例外
	 */
    public static void error(String msg, Throwable tt) {
    	if(level <= Log.ERROR) {
    		Log.e(getTag(), msg, tt);
    	}
    }

    /**
     * ログのヘッダ部分を返す。
     * @return ログのヘッダ部分
     */
    protected static String getTag() {
        Throwable t = new Throwable();
        String className = t.getStackTrace()[2].getClassName();
        String methodName = t.getStackTrace()[2].getMethodName();
        int lineNumber = t.getStackTrace()[2].getLineNumber();
        String sf = String.format("%s.%s(%d)", className, methodName, lineNumber);
        return sf;
    }

    /**
     * ログファイル名を設定する。
     * (androidで動いていない場合のみ有効)
     * @param logFileName フルパスではないファイル名
     */
    public static void setLogFileName(String logFileName) {
    	if(logFileName == null) {
    		// nullなら更新しない
    		return;
    	}
// android.util.Logの正式版を使うとコンパイルエラーになるから、リフレクションを使っている
//    	Log.setLogFileName(logFileName);

		try {
			Class<?>[] paramType = {String.class};
			invoke("android.util.Log", "setLogFileName", paramType, logFileName);
		} catch (Exception e) {
			// androidの場合
			// e.printStackTrace();
		}
    }

    /**
     * ログレベルを設定する。
     * （androidで動いていない場合のみ有効）
     * @param logLevel ログレベル
     */
    public static void setLogLevel(int logLevel) {
// android.util.Logの正式版を使うとコンパイルエラーになるから、リフレクションを使っている
//		Log.setOutputMinPriority(logLevel);
    	level = logLevel;
		try {
			Class<?>[] paramType = {int.class};
			invoke("android.util.Log", "setOutputMinPriority", paramType, logLevel);
		} catch (Exception e) {
			// androidの場合
			// e.printStackTrace();
		}
    }

    /**
     * ログレベルを設定する。
     * （androidで動いていない場合のみ有効）
     * @return ログレベル
     */
    public static int getLogLevel() {
    	return level;
    }

    private static Object invoke(String className, String methodName, Class<?>[] paramType, Object ... params) throws Exception{
		Class<?> c = Class.forName(className);
		Method method = c.getDeclaredMethod(methodName, paramType);
		Object result = method.invoke(null, params);
		return result;
    }

    /**
     * ログレベルを設定する。
     * （androidで動いていない場合のみ有効）
     * @param level ログレベル
     */
    public static void setLogLevel(String level) {
    	if(level == null) {
    		// nullなら更新しない
    		return;
    	}
    	setLogLevel(getPriority(level));
    }

    /**
     * ログレベルの文字列形式に変換する。
     * @param priority ログレベル
     * @return ログレベルの文字列形式
     */
	public static String getPriorityString(int priority) {
		String result = "INFO";
		switch(priority) {
        case Log.VERBOSE:
        	result = "VERBOSE";
        	break;
        case Log.DEBUG:
        	result = "DEBUG";
        	break;
        case Log.INFO:
        	result = "INFO";
        	break;
        case Log.WARN:
        	result = "WARN";
        	break;
        case Log.ERROR:
        	result = "ERROR";
        	break;
        }
		return result;
	}

    /**
     * ログレベルの数値形式に変換する。
     * @param priority ログレベル
     * @return ログレベルの数値形式
     */
	public static int getPriority(String priority) {
		int result = Log.INFO;
		switch(priority) {
        case "VERBOSE":
        case "2":
        	result = Log.VERBOSE;
        	break;
        case "DEBUG":
        case "3":
        	result = Log.DEBUG;
        	break;
        case "INFO":
        case "4":
        	result = Log.INFO;
        	break;
        case "WARN":
        case "5":
        	result = Log.WARN;
        	break;
        case "ERROR":
        case "6":
        	result = Log.ERROR;
        	break;
        }
		return result;
	}

}
