/*
 * Copyright (C) 2018 Gravity Engineering Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package example.test;

import java.util.Date;

import com.graveng.rci.Packet;
import com.graveng.rci.client.Rci;

/**
 * スタティックとインスタンスクラスを遠隔実行するクライアント側のサンプル
 */
public class Test0 {
   public static void main(String args[]) {
        try {
        	// RCIサーバのホスト名、ポート番号を指定
//            String server = "localhost";
            String server = "127.0.0.1";
//            String server = "0:0:0:0:0:0:0:1";
            int port = 5243;

            Rci rci = new Rci();
//	            rci.setUseEventListener(false);
            // ソケット接続
            rci.connect(server, port);
//	            rci.getInvoker().setRecvSleepMs(100);

            System.out.println("before " + new Date());
            System.out.println("鍵交換中");
            rci.invokeExchangeKey();
            System.out.println("after " + new Date());

            System.out.println("ログイン認証");
//            rci.invokeCertificateUser("山田太郎", "pass", "SHA-256", 0);

            // 以下の２つは同じユーザ名とパスワードによる認証
            rci.invokeCertificateUser("山田太郎", "01b613da484bee91c3f3806b52a6f40fd61ade874b5ffc0f62a2091cce38158b", "SHA-256", 0);
//            rci.invokeCertificateUser("山田太郎", "pass", "SHA-256", 2);

            // staticメソッドの実行
            System.out.println("スタティックメソッドの実行");
            Packet result = rci.invoke("static", "java.lang.System", "getProperty","user.dir");
            System.out.println("serverのuser.dir:"+result.getResult());

            System.out.println("インスタンスメソッドの実行");
            // インスタンス生成
            Packet dateResult = rci.invoke("new", "java.util.Date");

            // 生成したインスタンスのメソッド実行
            dateResult = rci.exec(dateResult, "getMonth");
            System.out.println("今月-1：" + dateResult.getResult());

            Packet hashMap = rci.invoke("new", "java.util.HashMap");
            rci.exec(hashMap, "put", "age", 21);
            int number = 0;
            number = rci.exec(hashMap, "get", "age").toResult(number);
            System.out.println("serverのHashMapに設定した値:"+number);

            // インスタンスの破棄(メソッド名以降がない場合はインスタンスを破棄する)
            rci.exec(dateResult);
            rci.exec(hashMap);

            // ソケット切断
            rci.disconnect();

        }
        catch (Exception e) {
        	e.printStackTrace();
        	System.exit(3);
        }
    }
}
